@@ -37,6 +37,7 @@
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <locale.h>
 
 #include <libgnomevfs/gnome-vfs-mime.h>
 
@@ -45,6 +46,7 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-module-shared.h>
+#include <libgnomevfs/gnome-vfs-i18n.h>
 
 #define LINESIZE 1024
 #define CDE_ICON_NAME_CACHE "/.gnome/g-cdeiconnames"
@@ -60,16 +62,45 @@ struct _DtFile {
 
 static GnomeVFSMethod *parent_method = NULL;
 
+static gchar *
+find_title_for_menu (gchar *name)
+{
+	GSList *filename;
+	char line[LINESIZE];
+	gboolean found=FALSE;
+	char *title="\0";
+    FILE *file = NULL;
+    gchar *tmp=NULL;
+
+    for (filename = cdemenufiles; filename !=NULL; filename=filename->next){
+		file = fopen((gchar*)filename->data,"r");
+		while (fgets(line,LINESIZE,file) != NULL){
+            if (strstr(line,"f.menu")  && strstr(line, name)) {
+				tmp = strstr (line, "f.menu");
+				*tmp = '\0';
+				title = strdup (line);
+				found = TRUE;
+				break;
+			}
+		}
+		fclose (file);
+		file = NULL;
+		if (found) break;
+	}
+	return title;
+}
+
 static FILE *
 open_and_find_pointer_to_menu(gchar *name)
 {
 	GSList *filename;
 	gchar line[LINESIZE];
-	FILE *file = NULL;
+	FILE *filesave = NULL, *file = NULL;
 	gboolean found = FALSE;
 	gchar *tmp=NULL;
 	
 	for (filename = cdemenufiles; filename !=NULL; filename=filename->next){
+		found = FALSE;
 		file = fopen((gchar*)filename->data,"r");
 		while (fgets(line,LINESIZE,file) != NULL){
 			if (strstr(line,"Menu") == line) {
@@ -82,11 +113,16 @@ open_and_find_pointer_to_menu(gchar *name)
 				}
 			}
 		}
-		if (found) break;
-		fclose(file);
-		file=NULL;
+		if (found) {
+			if (filesave !=NULL) fclose(filesave);
+			filesave = file;
+			if (!strcmp(name, "DtRootMenu")) break;
+		} else {
+			fclose(file);
+			file=NULL;
+		}
 	}
-	return file;
+	return filesave;
 }
 
 static char *
@@ -105,8 +141,13 @@ expand_env_vars(char *s)
 				tmp2 = tmp+(strlen(tmp))-1; *tmp2='\0';
 			}
 			tmp3 = g_getenv(tmp);
+			if (!tmp3 && strstr(tmp, "LC_CTYPE"))
+				/* It's not always the case $LC_CTYPE or $LANG is set */
+				tmp3 = (char*)setlocale(LC_CTYPE, NULL);
 			g_free(*token);
-			*token = g_strdup(tmp3);
+			/* don't do g_strdup (s?s1:s2) as that doesn't work with
+			   certain gcc 2.96 versions */
+			*token = tmp3 ? g_strdup(tmp3) : g_strdup("");
 		}
 	}
 	expanded = g_strjoinv("/", tokens);
@@ -118,25 +159,35 @@ expand_env_vars(char *s)
 static char*
 get_icon_for_menu (char *name) 
 {
-	if (!strcmp(name,"Applications")) 
+	/* Translate exactly the same in CDE's sys.dtwmrc file of the locale */
+	if (!strcmp(name, _("Applications")))
 		return ("/usr/dt/appconfig/icons/C/Dtapps.m.pm");
-	if (!strcmp(name,"Cards")) 
+	/* Translate  exactly the same in CDE's sys.dtwmrc file of the locale */
+	if (!strcmp(name, _("Cards")))
 		return ("/usr/dt/appconfig/icons/C/SDtCard.m.pm");
-	if (!strcmp(name,"Files")) 
+	/* Translate  exactly the same in CDE's sys.dtwmrc file of the locale */
+	if (!strcmp(name, _("Files")))
 		return ("/usr/dt/appconfig/icons/C/Dtdata.m.pm");
-	if (!strcmp(name,"Folders")) 
+	/* Translate  exactly the same in CDE's sys.dtwmrc file of the locale */
+	if (!strcmp(name, _("Folders")))
 		return ("/usr/dt/appconfig/icons/C/DtdirB.m.pm");
-	if (!strcmp(name,"Help")) 
+	/* Translate  exactly the same in CDE's sys.dtwmrc file of the locale */
+	if (!strcmp(name, _("Help")))
 		return ("/usr/dt/appconfig/icons/C/Dthelp.m.pm");
-	if (!strcmp(name,"Hosts")) 
+	/* Translate  exactly the same in CDE's sys.dtwmrc file of the locale */
+	if (!strcmp(name, _("Hosts")))
 		return ("/usr/dt/appconfig/icons/C/Dtterm.m.pm");
-	if (!strcmp(name,"Links")) 
+	/* Translate  exactly the same in CDE's sys.dtwmrc file of the locale */
+	if (!strcmp(name, _("Links")))
 		return ("/usr/dt/appconfig/icons/C/SDturlweb.m.pm");
-	if (!strcmp(name,"Mail")) 
+	/* Translate  exactly the same in CDE's sys.dtwmrc file of the locale */
+	if (!strcmp(name, _("Mail")))
 		return ("/usr/dt/appconfig/icons/C/Dtmail.m.pm");
-	if (!strcmp(name,"Tools")) 
+	/* Translate  exactly the same in CDE's sys.dtwmrc file of the locale */
+	if (!strcmp(name, _("Tools")))
 		return ("/usr/dt/appconfig/icons/C/SDtGears.m.pm");
-	if (!strcmp(name,"Windows")) 
+	/* Translate exactly the same in CDE's sys.dtwmrc file of the locale */
+	if (!strcmp(name, _("Windows")))
 		return ("/usr/dt/appconfig/icons/C/DtDtwm.m.pm");
 	return ("");
 }
@@ -152,7 +203,8 @@ get_icon_for_action(char *action)
 
 	char *iconsearchpath[] = {"/usr/dt/appconfig/icons/$LC_CTYPE",
                        "/etc/dt/appconfig/icons/$LC_CTYPE",
-                       "$HOME/.dt/icons", NULL };
+                       "$HOME/.dt/icons",
+                       "/usr/dt/appconfig/icons/C", NULL};
 
 	/*if action has arg strip the arg*/
 	if ((tmp=strchr(action,'\"'))) {*tmp='\0'; tmp=NULL;}
@@ -207,7 +259,7 @@ do_open (GnomeVFSMethod *method,
 	FILE *file;
 	gchar *tmp, *tmp2, *end;
 	gchar line[LINESIZE];
-	gchar *name, *exec=NULL, *icon=NULL;
+	gchar *name, *title, *exec=NULL, *icon=NULL;
 	gchar *menuname = g_path_get_basename(g_path_get_dirname(uri->text));
 	DtFile *dtfile = g_new0(DtFile, 1);
 
@@ -230,47 +282,120 @@ do_open (GnomeVFSMethod *method,
 	if(!file) return GNOME_VFS_ERROR_NOT_FOUND;
 
 	if (strcmp(name,".directory") == 0) {
-		while (fgets(line,LINESIZE,file) != NULL){
-			if (line[0] == '#') continue;
-			else if (line[0] == '}') break;
-			else if (strstr(line,"f.title")) {
-				tmp = strchr(line,'\"'); tmp ++;
-				tmp2 = strchr(tmp,'\"'); if(tmp2) *tmp2='\0';
-				dtfile->contents= g_strdup_printf("[Desktop "
-					"Entry]\nName=%s\nComment=%s\nIcon=%s\nType=Directory\n",
-					 tmp,tmp,get_icon_for_menu(tmp));
-				/* Title is found, dont look for secondary title*/
-				break;
-			}
-		}
+				char *utf8_name = NULL;
+				if (!strcmp (menuname, "DtRootMenu"))
+					title = g_strdup_printf ("\"%s\"",menuname);
+				else title = find_title_for_menu(menuname);
+
+				tmp = strchr(title,'\"'); 
+                                if(!tmp) {
+                                        title = g_strstrip (title);
+                                        tmp = title;
+                                }
+                                else {
+                                        tmp ++;
+                                        tmp2 = strchr(tmp,'\"');
+                                        if(tmp2) *tmp2='\0';
+                                }
+
+				utf8_name = g_locale_to_utf8 (tmp, -1,
+							      NULL, NULL,
+							      NULL);
+				/* fallback to avoid crash */
+				if (utf8_name == NULL)
+					utf8_name = g_strdup (tmp);
+
+				dtfile->contents = g_strdup_printf 
+					("[Desktop Entry]\n"
+					 "Encoding=UTF-8\n"
+					 /* Note that we include the utf-8
+					  * translated name without a language
+					  * specifier, but that's OK, it will
+					  * work */
+					 "Name=%s\n"
+					 "Comment=\n"
+					 "Icon=%s\n"
+					 "Type=Directory\n",
+					 utf8_name, 
+					 get_icon_for_menu(utf8_name));
+
+				g_free (utf8_name);
+				g_free (title);
 	} else {
 		while (fgets(line,LINESIZE,file) != NULL){
 			if (line[0] == '#') continue;
 			else if (line[0] == '}') break;
 			else if (strstr(line,name)){
 				if((tmp = strstr(line,"f.action")) != NULL){
+					char *utf8_name = NULL;
+
 					tmp+=8;
 					end = strchr(tmp,'\n');
 					if (end) *end='\0';
 					exec=g_strdup_printf("dtaction %s",tmp);
 					icon=get_icon_for_action(tmp);
 					if (!icon) icon=g_strdup("NONE");
-					dtfile->contents= g_strdup_printf("[Desktop Entry]\nName=%s"
-								"\nComment=%s\nExec=%s\nIcon=%s\nTerminal=0\n"
-								"Type=Application\n", name, name, exec, icon);
+
+					utf8_name = g_locale_to_utf8 (name, -1,
+								      NULL,
+								      NULL,
+								      NULL);
+					/* fallback to avoid crash */
+					if (utf8_name == NULL)
+						utf8_name = g_strdup (name);
+
+					dtfile->contents= g_strdup_printf
+						("[Desktop Entry]\n"
+						 "Encoding=UTF-8\n"
+						 /* Note that we include the
+						  * utf-8 translated name
+						  * without a language
+						  * specifier, but that's
+						  * OK, it will work */
+						 "Name=%s\n"
+						 "Comment=\n"
+						 "Exec=%s\n"
+						 "Icon=%s\n"
+						 "Terminal=0\n"
+						 "Type=Application\n",
+						  utf8_name,
+						 exec, icon);
+					g_free (utf8_name);
 					g_free(icon);
 				} else if ((tmp = strstr(line,"f.exec"))!=NULL){
+					char *utf8_name = NULL;
+
 					tmp+=6;
 					tmp = g_strstrip(tmp);
 					/* strip off quotes */
 					if (*tmp == '\"') tmp++;
 					end = tmp+(strlen(tmp))-1;
 					if (*end == '\"') *end = '\0';
-					dtfile->contents= g_strdup_printf("[Desktop Entry]\nName=%s"
-								"\nComment=%s\nExec=%s\n"
-								"Icon=\n"
-								"Terminal=0\n"
-			  		  			"Type=Application\n", name, name, tmp);
+
+					utf8_name = g_locale_to_utf8 (name, -1,
+								      NULL,
+								      NULL,
+								      NULL);
+					/* fallback to avoid crash */
+					if (utf8_name == NULL)
+						utf8_name = g_strdup (name);
+
+					dtfile->contents= g_strdup_printf
+						("[Desktop Entry]\n"
+						 "Encoding=UTF-8\n"
+						 /* Note that we include the
+						  * utf-8 translated name
+						  * without a language
+						  * specifier, but that's
+						  * OK, it will work */
+						 "Name=%s\n"
+						 "Comment=\n"
+						 "Exec=%s\n"
+						 "Icon=\n"
+						 "Terminal=0\n"
+						 "Type=Application\n",
+						  utf8_name, tmp);
+					g_free (utf8_name);
 				}
 			}
 		}
@@ -368,7 +493,7 @@ do_read_directory (GnomeVFSMethod *method,
 					|| strstr(line,"UpdateWorkspaceMenu"))) {
 					file_info->name = g_strdup("bogus.desktop");
 					file_info->type = GNOME_VFS_FILE_TYPE_REGULAR;
- 					break;
+ 					continue;
 			}
 			else if (strstr(line,"f.action") || strstr(line,"f.exec")) {
 					tmp = strchr(line,'\"'); tmp ++;
@@ -498,9 +623,14 @@ create_cde_icon_name_cache (void)
 	mode_t old_mask;
 	char *icon_name_cache_path;
 	
-	if ((t = fork ()) < 0) g_error ("Unable to fork.");
-	else if (t) wait(NULL);
-	else {
+	if ((t = fork ()) < 0) {
+		g_error ("Unable to fork.");
+	} else if (t) {
+		/* On a Solaris box, waitpid() fails with an interrupted system call.
+		   Hence, loop till it succeeds. Avoids zombies
+		*/
+		while ((waitpid (t,NULL,0) == -1) && errno == EINTR);
+	} else {
 		icon_name_cache_path = g_strconcat (g_get_home_dir (), CDE_ICON_NAME_CACHE, NULL);
 		old_mask = umask(033);
 		fd = open (icon_name_cache_path, O_WRONLY | O_CREAT | O_TRUNC, 0644);
